/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl;

import static org.ow2.petals.component.framework.junit.JbiConstants.CDK_NAMESPACE_PREFIX;
import static org.ow2.petals.component.framework.junit.JbiConstants.CDK_NAMESPACE_URI;
import static org.ow2.petals.component.framework.junit.JbiConstants.JBI_NAMESPACE_PREFIX;
import static org.ow2.petals.component.framework.junit.JbiConstants.JBI_NAMESPACE_URI;
import static org.ow2.petals.component.framework.junit.JbiConstants.XSI_NAMESPACE_PREFIX;
import static org.ow2.petals.component.framework.junit.JbiConstants.XSI_NAMESPACE_URI;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.io.IOUtils;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.junit.impl.ServiceConfiguration.ServiceType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public class ServiceUnitJbiGenerator {

    private static final String SERVICE_NAMESPACE_PREFIX = "srvNs";

    private static final String INTERFACE_NAMESPACE_PREFIX = "itfNs";

    private final ServiceConfiguration serviceConfiguration;

    private final File installationRoot;

    public ServiceUnitJbiGenerator(final ServiceConfiguration serviceConfiguration, final File installationRoot) {
        assert serviceConfiguration != null;
        assert installationRoot != null;
        this.serviceConfiguration = serviceConfiguration;
        this.installationRoot = installationRoot;
    }

    /**
     * Generate the jbi.xml file.
     * 
     * @param multiservice
     *            If true, services dependencies of the
     *            {@link ServiceConfiguration} will be generate in the same
     *            jbi.xml file.
     * @return
     */
    public final File generate(final boolean multiservice) {

        final Document document = DocumentBuilders.newDocument();
        document.setXmlVersion("1.0");
        document.setXmlStandalone(true);

        final Map<String, String> jbiEltAttr = new HashMap<String, String>();
        jbiEltAttr.put("version", "1.0");
        final Element jbiElement = ServiceUnitJbiGenerator.addElement(document, document, JBI_NAMESPACE_URI,
                JBI_NAMESPACE_PREFIX + ":jbi", jbiEltAttr);
        jbiElement.setAttribute(XMLConstants.XMLNS_ATTRIBUTE + ":" + XSI_NAMESPACE_PREFIX, XSI_NAMESPACE_URI);

        final Map<String, String> servicesEltAttr = new HashMap<String, String>();
        servicesEltAttr.put("binding-component", "true");

        final Element servicesElement = addElement(document, jbiElement, JBI_NAMESPACE_URI, "jbi:services",
                servicesEltAttr);

        this.generateServiceTypeElement(document, servicesElement, serviceConfiguration);

        final ServiceConfiguration[] dependencies = this.serviceConfiguration.getServiceConfigurationDependencies();
        if (dependencies != null) {
            for (final ServiceConfiguration s : dependencies) {
                    this.generateServiceTypeElement(document, servicesElement, s);
            }
        }

        for (final QName servicesSectionParemeter : serviceConfiguration.getServicesSectionParameterNames()) {
            ServiceUnitJbiGenerator.addElement(document, servicesElement, servicesSectionParemeter.getNamespaceURI(),
                    servicesSectionParemeter.getLocalPart(),
                    serviceConfiguration.getServicesSectionParameter(servicesSectionParemeter));
        }

        final File serviceUnitMetaInfDir = new File(installationRoot, "META-INF");
        serviceUnitMetaInfDir.mkdirs();
        final File jbiDumpTarget = new File(serviceUnitMetaInfDir, "jbi.xml");
        try {
            final OutputStream fos = new FileOutputStream(jbiDumpTarget);
            try {
                final Result result = new StreamResult(fos);
                final Source domSource = new DOMSource(document);
                final Transformer transformer = Transformers.takeTransformer();
                try {
                    transformer.transform(domSource, result);
                    return jbiDumpTarget;
                } catch (final TransformerException e) {
                    throw new UncheckedException("Impossible to generate the JBI file", e);
                } finally {
                    Transformers.releaseTransformer(transformer);
                }
            } finally {
                IOUtils.closeQuietly(fos);
            }
        } catch (final FileNotFoundException e) {
            throw new UncheckedException("Impossible to generate the JBI file", e);
        }
    }

    /**
     * Generate provide or consume element
     * @param document
     * @param servicesElement
     * @param serviceConfiguration
     */
    private void generateServiceTypeElement(final Document document, final Element servicesElement,
            final ServiceConfiguration serviceConfiguration) {
        final String serviceTypeElementName;
        if (ServiceType.CONSUME.equals(serviceConfiguration.getServiceType())) {
            serviceTypeElementName = "jbi:consumes";
        } else {
            serviceTypeElementName = "jbi:provides";
        }
        // Create attributes
        final Map<String, String> serviceTypeEltAttr = new HashMap<String, String>();
        serviceTypeEltAttr.put("endpoint-name", serviceConfiguration.getEndpointName());
        serviceTypeEltAttr.put("interface-name", INTERFACE_NAMESPACE_PREFIX + ":"
                + serviceConfiguration.getInterfaceQName().getLocalPart());
        serviceTypeEltAttr.put("service-name", SERVICE_NAMESPACE_PREFIX + ":"
                + serviceConfiguration.getServiceQName().getLocalPart());
        serviceTypeEltAttr.put(XMLConstants.XMLNS_ATTRIBUTE + ":" + SERVICE_NAMESPACE_PREFIX,
                serviceConfiguration.getServiceQName().getNamespaceURI());
        serviceTypeEltAttr.put(XMLConstants.XMLNS_ATTRIBUTE + ":" + INTERFACE_NAMESPACE_PREFIX,
                serviceConfiguration.getInterfaceQName().getNamespaceURI());
        final Element providesElement = addElement(document, servicesElement, JBI_NAMESPACE_URI,
                serviceTypeElementName, serviceTypeEltAttr);
        
        final long timeout = serviceConfiguration.getTimeout();
        ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                "timeout", "" + timeout);
        
        final String operation = serviceConfiguration.getOperation();
        if (operation != null) {
            ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                    "operation",
                    SERVICE_NAMESPACE_PREFIX + ":" + operation);
        }

        final MEPType mep = serviceConfiguration.getMEP();
        if (mep != null) {
            ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                    "mep", mep.value());
        } else if (ServiceType.CONSUME.equals(serviceConfiguration.getServiceType())) {
            ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                    "mep",
                    Collections.singletonMap(XSI_NAMESPACE_PREFIX + ":nil", "true"));
        }

        if (ServiceType.PROVIDE.equals(serviceConfiguration.getServiceType())) {
            if (serviceConfiguration.getServiceDescription() != null) {
                try {
                    final File wsdl = new File(serviceConfiguration.getServiceDescription().toURI());
                    ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                            "wsdl", wsdl.getName());
                } catch (final URISyntaxException e) {
                    throw new UncheckedException(e);
                }
            } else {
                ServiceUnitJbiGenerator.addElement(document, providesElement, CDK_NAMESPACE_URI,
                        "wsdl");
            }
        }

        //append service parameters
        for (final QName specificParameterQName : serviceConfiguration.getParameterNames()) {
            ServiceUnitJbiGenerator.addElement(
                    document, providesElement,
                    specificParameterQName.getNamespaceURI(),
                    specificParameterQName.getLocalPart(),
                    serviceConfiguration.getParameter(specificParameterQName));
        }
    }

    private static final Element addElement(final Document document, final Node parentNode, final String namespaceUri,
            final String qualifiedName) {
        return ServiceUnitJbiGenerator.addElement(document, parentNode, namespaceUri, qualifiedName, null,
                new HashMap<String, String>());
    }

    private static final Element addElement(final Document document, final Node parentNode, final String namespaceUri,
            final String qualifiedName, final Map<String, String> attributes) {
        return ServiceUnitJbiGenerator.addElement(document, parentNode, namespaceUri, qualifiedName, null, attributes);
    }

    private static final Element addElement(final Document document, final Node parentNode, final String namespaceUri,
            final String qualifiedName, final String value) {
        return ServiceUnitJbiGenerator.addElement(document, parentNode, namespaceUri, qualifiedName, value,
                new HashMap<String, String>());
    }

    private static final Element addElement(final Document document, final Node parentNode, final String namespaceUri,
            final String qualifiedName, final String value, final Map<String, String> attributes) {
        final Element newElement = document.createElementNS(namespaceUri, qualifiedName);
        if (value != null) {
            newElement.setTextContent(value);
        }
        for (final Entry<String, String> e : attributes.entrySet()) {
            newElement.setAttribute(e.getKey(), e.getValue());
        }
        parentNode.appendChild(newElement);
        return newElement;
    }

}
