/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.message;

import static org.mockito.Mockito.when;

import java.io.InputStream;
import java.io.StringReader;
import java.net.URI;
import java.util.UUID;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.mockito.Mockito;
import org.ow2.petals.commons.log.FlowAttributes;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.junit.RequestMessage;
import org.ow2.petals.component.framework.junit.impl.ServiceConfiguration;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * A message exchange as request to send to a service provider
 * 
 * @author Marc Jambert - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 */
public class WrappedRequestToProviderMessage extends AbstractWrappedToComponentMessage implements RequestMessage {
    
    /**
     * Create a message exchange with attributes for MONIT traces set to random values
     * 
     * @param serviceConfiguration
     *            The configuration of the target service provider
     * @param operation
     *            The operation to invoke
     * @param mep
     *            The MEP to use
     * @param inMessageContentStream
     *            The XML payload. If <code>null</code> an empty {@link Source} is set.
     */
    public WrappedRequestToProviderMessage(final ServiceConfiguration serviceConfiguration, final QName operation,
            final URI mep, final InputStream inMessageContentStream) {
        this(serviceConfiguration, operation, mep, new StreamSource(inMessageContentStream), new FlowAttributes(UUID.randomUUID()
                .toString(), UUID.randomUUID().toString()));
    }
    
    public WrappedRequestToProviderMessage(final ServiceConfiguration serviceConfiguration, final QName operation,
            final URI mep, final String inMessageContent) {
        this(serviceConfiguration, operation, mep, new StreamSource(new StringReader(inMessageContent)), new FlowAttributes(UUID.randomUUID()
                .toString(), UUID.randomUUID().toString()));
    }

    public WrappedRequestToProviderMessage(final ServiceConfiguration serviceConfiguration, final QName operation,
            final URI mep) {
        this(serviceConfiguration, operation, mep, null,
                new FlowAttributes(UUID.randomUUID().toString(), UUID.randomUUID().toString()));
    }

    /**
     * 
     * @param serviceConfiguration
     *            The configuration of the target service provider
     * @param operation
     *            The operation to invoke
     * @param mep
     *            The MEP to use
     * @param inContent
     *            The XML payload. If <code>null</code> an empty {@link Source} is set.
     * @param flowAttributes
     *            Attributes for MONIT traces
     */
    public WrappedRequestToProviderMessage(final ServiceConfiguration serviceConfiguration, final QName operation,
            final URI mep, final Source inContent, final FlowAttributes flowAttributes) {
        super(serviceConfiguration, operation, mep, flowAttributes);

        try {
            this.messageExchange.setRole(Role.CONSUMER);
            final NormalizedMessage inNm = this.messageExchange.createMessage();
            inNm.setContent(inContent);
            this.messageExchange.setMessage(inNm, Exchange.IN_MESSAGE_NAME);
            this.messageExchange.setRole(Role.PROVIDER);
        } catch (final MessagingException e) {
            throw new UncheckedException("Impossible to create a message exchange", e);
        }
    }

    public WrappedRequestToProviderMessage(final MessageExchange messageExchange) {
        super(messageExchange);

        final ServiceEndpoint mockServiceEndpoint = Mockito.mock(ServiceEndpoint.class);
        final ServiceEndpoint endpoint = messageExchange.getEndpoint();
        final String endpointName;
        if (endpoint != null) {
            endpointName = endpoint.getEndpointName();
        } else {
            endpointName = null;
        }
        final QName serviceName = messageExchange.getService();
        when(mockServiceEndpoint.getEndpointName()).thenReturn(endpointName);
        when(mockServiceEndpoint.getServiceName()).thenReturn(serviceName);
        messageExchange.setEndpoint(mockServiceEndpoint);

        this.messageExchange.setRole(Role.PROVIDER);
    }
    
	@Override
    public Source getPayload() {
        return getPayload(PetalsMessageExchange.IN_MSG);
    }

    
}
